# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Resource definitions for Cloud Platform Apis generated from apitools."""

import enum


BASE_URL = 'https://aiplatform.googleapis.com/v1/'
DOCS_URL = 'https://cloud.google.com/vertex-ai/'


class Collections(enum.Enum):
  """Collections for all supported apis."""

  OPERATIONS = (
      'operations',
      '{+name}',
      {
          '':
              'operations/{operationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS = (
      'projects.locations',
      'projects/{projectsId}/locations/{locationsId}',
      {},
      ['projectsId', 'locationsId'],
      True
  )
  PROJECTS_LOCATIONS_BATCHPREDICTIONJOBS = (
      'projects.locations.batchPredictionJobs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'batchPredictionJobs/{batchPredictionJobsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_CACHEDCONTENTS = (
      'projects.locations.cachedContents',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/cachedContents/'
              '{cachedContentsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_CUSTOMJOBS = (
      'projects.locations.customJobs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/customJobs/'
              '{customJobsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATASETS = (
      'projects.locations.datasets',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/datasets/'
              '{datasetsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATASETS_ANNOTATIONSPECS = (
      'projects.locations.datasets.annotationSpecs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/datasets/'
              '{datasetsId}/annotationSpecs/{annotationSpecsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DATASETS_DATASETVERSIONS = (
      'projects.locations.datasets.datasetVersions',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/datasets/'
              '{datasetsId}/datasetVersions/{datasetVersionsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_DEPLOYMENTRESOURCEPOOLS = (
      'projects.locations.deploymentResourcePools',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'deploymentResourcePools/{deploymentResourcePoolsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_ENDPOINTS = (
      'projects.locations.endpoints',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/endpoints/'
              '{endpointsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_ENDPOINTS_OPERATIONS = (
      'projects.locations.endpoints.operations',
      'projects/{projectsId}/locations/{locationsId}/endpoints/{endpointsId}/'
      'operations/{operationsId}',
      {},
      ['projectsId', 'locationsId', 'endpointsId', 'operationsId'],
      True
  )
  PROJECTS_LOCATIONS_FEATUREGROUPS = (
      'projects.locations.featureGroups',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/featureGroups/'
              '{featureGroupsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_FEATUREGROUPS_FEATURES = (
      'projects.locations.featureGroups.features',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/featureGroups/'
              '{featureGroupsId}/features/{featuresId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_FEATUREONLINESTORES = (
      'projects.locations.featureOnlineStores',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'featureOnlineStores/{featureOnlineStoresId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_FEATUREONLINESTORES_FEATUREVIEWS = (
      'projects.locations.featureOnlineStores.featureViews',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'featureOnlineStores/{featureOnlineStoresId}/featureViews/'
              '{featureViewsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_FEATUREONLINESTORES_FEATUREVIEWS_FEATUREVIEWSYNCS = (
      'projects.locations.featureOnlineStores.featureViews.featureViewSyncs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'featureOnlineStores/{featureOnlineStoresId}/featureViews/'
              '{featureViewsId}/featureViewSyncs/{featureViewSyncsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_FEATURESTORES = (
      'projects.locations.featurestores',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/featurestores/'
              '{featurestoresId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_FEATURESTORES_ENTITYTYPES = (
      'projects.locations.featurestores.entityTypes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/featurestores/'
              '{featurestoresId}/entityTypes/{entityTypesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_FEATURESTORES_ENTITYTYPES_FEATURES = (
      'projects.locations.featurestores.entityTypes.features',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/featurestores/'
              '{featurestoresId}/entityTypes/{entityTypesId}/features/'
              '{featuresId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_HYPERPARAMETERTUNINGJOBS = (
      'projects.locations.hyperparameterTuningJobs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'hyperparameterTuningJobs/{hyperparameterTuningJobsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_INDEXENDPOINTS = (
      'projects.locations.indexEndpoints',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/indexEndpoints/'
              '{indexEndpointsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_INDEXENDPOINTS_OPERATIONS = (
      'projects.locations.indexEndpoints.operations',
      'projects/{projectsId}/locations/{locationsId}/indexEndpoints/'
      '{indexEndpointsId}/operations/{operationsId}',
      {},
      ['projectsId', 'locationsId', 'indexEndpointsId', 'operationsId'],
      True
  )
  PROJECTS_LOCATIONS_INDEXES = (
      'projects.locations.indexes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/indexes/'
              '{indexesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_INDEXES_OPERATIONS = (
      'projects.locations.indexes.operations',
      'projects/{projectsId}/locations/{locationsId}/indexes/{indexesId}/'
      'operations/{operationsId}',
      {},
      ['projectsId', 'locationsId', 'indexesId', 'operationsId'],
      True
  )
  PROJECTS_LOCATIONS_METADATASTORES = (
      'projects.locations.metadataStores',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/metadataStores/'
              '{metadataStoresId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_METADATASTORES_ARTIFACTS = (
      'projects.locations.metadataStores.artifacts',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/metadataStores/'
              '{metadataStoresId}/artifacts/{artifactsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_METADATASTORES_CONTEXTS = (
      'projects.locations.metadataStores.contexts',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/metadataStores/'
              '{metadataStoresId}/contexts/{contextsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_METADATASTORES_EXECUTIONS = (
      'projects.locations.metadataStores.executions',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/metadataStores/'
              '{metadataStoresId}/executions/{executionsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_METADATASTORES_METADATASCHEMAS = (
      'projects.locations.metadataStores.metadataSchemas',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/metadataStores/'
              '{metadataStoresId}/metadataSchemas/{metadataSchemasId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_MODELDEPLOYMENTMONITORINGJOBS = (
      'projects.locations.modelDeploymentMonitoringJobs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'modelDeploymentMonitoringJobs/'
              '{modelDeploymentMonitoringJobsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_MODELS = (
      'projects.locations.models',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/models/'
              '{modelsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_MODELS_EVALUATIONS = (
      'projects.locations.models.evaluations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/models/'
              '{modelsId}/evaluations/{evaluationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_MODELS_EVALUATIONS_SLICES = (
      'projects.locations.models.evaluations.slices',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/models/'
              '{modelsId}/evaluations/{evaluationsId}/slices/{slicesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_MODELS_OPERATIONS = (
      'projects.locations.models.operations',
      'projects/{projectsId}/locations/{locationsId}/models/{modelsId}/'
      'operations/{operationsId}',
      {},
      ['projectsId', 'locationsId', 'modelsId', 'operationsId'],
      True
  )
  PROJECTS_LOCATIONS_NASJOBS = (
      'projects.locations.nasJobs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/nasJobs/'
              '{nasJobsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_NASJOBS_NASTRIALDETAILS = (
      'projects.locations.nasJobs.nasTrialDetails',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/nasJobs/'
              '{nasJobsId}/nasTrialDetails/{nasTrialDetailsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_NOTEBOOKEXECUTIONJOBS = (
      'projects.locations.notebookExecutionJobs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'notebookExecutionJobs/{notebookExecutionJobsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_NOTEBOOKRUNTIMETEMPLATES = (
      'projects.locations.notebookRuntimeTemplates',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'notebookRuntimeTemplates/{notebookRuntimeTemplatesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_NOTEBOOKRUNTIMES = (
      'projects.locations.notebookRuntimes',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'notebookRuntimes/{notebookRuntimesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_OPERATIONS = (
      'projects.locations.operations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/operations/'
              '{operationsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_PERSISTENTRESOURCES = (
      'projects.locations.persistentResources',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'persistentResources/{persistentResourcesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_PIPELINEJOBS = (
      'projects.locations.pipelineJobs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/pipelineJobs/'
              '{pipelineJobsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_RAGCORPORA = (
      'projects.locations.ragCorpora',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/ragCorpora/'
              '{ragCorporaId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_RAGCORPORA_RAGFILES = (
      'projects.locations.ragCorpora.ragFiles',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/ragCorpora/'
              '{ragCorporaId}/ragFiles/{ragFilesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_REASONINGENGINES = (
      'projects.locations.reasoningEngines',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'reasoningEngines/{reasoningEnginesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_REASONINGENGINES_SESSIONS = (
      'projects.locations.reasoningEngines.sessions',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'reasoningEngines/{reasoningEnginesId}/sessions/{sessionsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_SCHEDULES = (
      'projects.locations.schedules',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/schedules/'
              '{schedulesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_SEMANTICGOVERNANCEPOLICIES = (
      'projects.locations.semanticGovernancePolicies',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'semanticGovernancePolicies/{semanticGovernancePoliciesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_SEMANTICGOVERNANCEPOLICIES_OPERATIONS = (
      'projects.locations.semanticGovernancePolicies.operations',
      'projects/{projectsId}/locations/{locationsId}/'
      'semanticGovernancePolicies/{semanticGovernancePoliciesId}/operations/'
      '{operationsId}',
      {},
      ['projectsId', 'locationsId', 'semanticGovernancePoliciesId', 'operationsId'],
      True
  )
  PROJECTS_LOCATIONS_SEMANTICGOVERNANCEPOLICYENGINE = (
      'projects.locations.semanticGovernancePolicyEngine',
      'projects/{projectsId}/locations/{locationsId}/'
      'semanticGovernancePolicyEngine',
      {},
      ['projectsId', 'locationsId'],
      True
  )
  PROJECTS_LOCATIONS_SPECIALISTPOOLS = (
      'projects.locations.specialistPools',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/specialistPools/'
              '{specialistPoolsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_STUDIES = (
      'projects.locations.studies',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/studies/'
              '{studiesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_STUDIES_TRIALS = (
      'projects.locations.studies.trials',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/studies/'
              '{studiesId}/trials/{trialsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_TENSORBOARDS = (
      'projects.locations.tensorboards',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/tensorboards/'
              '{tensorboardsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_TENSORBOARDS_EXPERIMENTS = (
      'projects.locations.tensorboards.experiments',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/tensorboards/'
              '{tensorboardsId}/experiments/{experimentsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_TENSORBOARDS_EXPERIMENTS_RUNS = (
      'projects.locations.tensorboards.experiments.runs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/tensorboards/'
              '{tensorboardsId}/experiments/{experimentsId}/runs/{runsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_TENSORBOARDS_EXPERIMENTS_RUNS_TIMESERIES = (
      'projects.locations.tensorboards.experiments.runs.timeSeries',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/tensorboards/'
              '{tensorboardsId}/experiments/{experimentsId}/runs/{runsId}/'
              'timeSeries/{timeSeriesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_TRAININGPIPELINES = (
      'projects.locations.trainingPipelines',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/'
              'trainingPipelines/{trainingPipelinesId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_TUNINGJOBS = (
      'projects.locations.tuningJobs',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/tuningJobs/'
              '{tuningJobsId}',
      },
      ['name'],
      True
  )
  PUBLISHERS = (
      'publishers',
      'publishers/{publishersId}',
      {},
      ['publishersId'],
      True
  )
  PUBLISHERS_MODELS = (
      'publishers.models',
      '{+name}',
      {
          '':
              'publishers/{publishersId}/models/{modelsId}',
      },
      ['name'],
      True
  )

  def __init__(self, collection_name, path, flat_paths, params,
               enable_uri_parsing):
    self.collection_name = collection_name
    self.path = path
    self.flat_paths = flat_paths
    self.params = params
    self.enable_uri_parsing = enable_uri_parsing



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to cancel a fleet rollout."""

from googlecloudsdk.api_lib.container.fleet import client
from googlecloudsdk.api_lib.container.fleet import types
from googlecloudsdk.api_lib.container.fleet import util
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import parser_arguments
from googlecloudsdk.calliope import parser_extensions
from googlecloudsdk.command_lib.container.fleet.rollouts import flags as rollout_flags
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io


_EXAMPLES = """
To cancel a rollout, run:

$ {command} ROLLOUT
"""


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Cancel(base.UpdateCommand):
  """Cancel a rollout."""

  detailed_help = {'EXAMPLES': _EXAMPLES}

  @classmethod
  def Args(cls, parser: parser_arguments.ArgumentInterceptor) -> None:
    """Registers flags for this command."""
    flags = rollout_flags.RolloutFlags(parser, cls.ReleaseTrack())
    flags.AddRolloutResourceArg()
    flags.AddAsync()

  def Run(self, args: parser_extensions.Namespace) -> types.Operation:
    """Runs the cancel command."""
    flag_parser = rollout_flags.RolloutFlagParser(
        args, release_track=self.ReleaseTrack()
    )
    fleet_client = client.FleetClient(release_track=self.ReleaseTrack())
    rollout_name = util.RolloutName(args)
    req = fleet_client.messages.GkehubProjectsLocationsRolloutsCancelRequest(
        name=rollout_name,
        cancelRolloutRequest=fleet_client.messages.CancelRolloutRequest(),
    )

    console_io.PromptContinue(
        message=f'You are about to cancel rollout [{rollout_name}].',
        cancel_on_no=True,
    )

    operation = fleet_client.CancelRollout(req)
    rollout_ref = util.RolloutRef(args)

    if flag_parser.Async():
      log.status.Print(
          f'Cancel in progress for Fleet rollout [{rollout_ref.SelfLink()}]'
      )
      return operation

    operation_client = client.OperationClient(
        release_track=self.ReleaseTrack()
    )
    completed_operation = operation_client.Wait(util.OperationRef(operation))
    log.status.Print(f'Cancelled Fleet rollout [{rollout_ref.SelfLink()}].')

    return completed_operation



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Manager for starting and stopping the local ECP HTTP Proxy."""

import atexit
import os
import secrets
import socket
import subprocess
import time
from typing import Optional

from googlecloudsdk.core import context_aware
from googlecloudsdk.core import execution_utils
from googlecloudsdk.core import log
from googlecloudsdk.core.util import encoding

_CLOUDSDK_ECP_HTTP_PROXY_PORT = 'CLOUDSDK_ECP_HTTP_PROXY_PORT'
_ECP_HTTP_PROXY_MANAGER_INSTANCE = None


class ECPProxyError(Exception):
  """Custom exception for errors related to the ECP proxy communication."""

  def __init__(self, message, original_exception=None):
    self.original_exception = original_exception
    super().__init__(
        f'{message}: {original_exception}' if original_exception else message
    )


def _find_free_port() -> int:
  """Dynamically finds and returns an available TCP port."""
  with socket.socket() as s:
    s.bind(('', 0))
    return s.getsockname()[1]


class _ECPHTTPProxyManager(object):
  """Manages the lifecycle of a local ECP HTTP proxy process."""

  def __init__(
      self, certificate_config_file_path: str, startup_timeout: int = 5
  ):
    self.certificate_config_file_path = certificate_config_file_path
    self.proxy_process = None
    self.proxy_host = 'localhost'
    self.nonce_token = secrets.token_hex(16)
    self.startup_timeout = startup_timeout

    # pylint:disable=g-import-not-at-top
    from googlecloudsdk.core import requests as core_requests

    self.gcloud_proxy_url = core_requests.GetProxyInfo()

    # Register a cleanup function to terminate the proxy process on exit.
    atexit.register(self.close)

    self.proxy_port = self._start_ecp_proxy_with_retries(
        self.startup_timeout, max_retries=1
    )
    encoding.SetEncodedValue(
        os.environ, _CLOUDSDK_ECP_HTTP_PROXY_PORT, str(self.proxy_port)
    )

  def _start_ecp_proxy_with_retries(
      self, timeout: int, max_retries: int = 1
  ) -> int:
    """Attempts to start the ECP Proxy, retrying on failure."""
    cert_config = context_aware.GetCertificateConfig(
        self.certificate_config_file_path
    )
    ecp_http_proxy = cert_config.get('libs', {}).get('ecp_http_proxy')
    if not ecp_http_proxy:
      raise ECPProxyError(
          'ECP HTTP proxy binary path is not specified in enterprise'
          ' certificate config file. Cannot use mTLS with ECP if the ECP HTTP'
          ' proxy binary does not exist. Please check the ECP configuration.'
          ' See `gcloud topic client-certificate` to learn more about ECP. \nIf'
          ' this error is unexpected either delete {} or generate a new'
          ' configuration with `$ gcloud auth enterprise-certificate-config'
          ' create --help` '.format(self.certificate_config_file_path)
      )

    for attempt in range(max_retries + 1):
      proxy_port = _find_free_port()
      self._start_ecp_proxy(
          ecp_http_proxy=ecp_http_proxy, proxy_port=proxy_port
      )

      try:
        self._wait_for_proxy(proxy_port=proxy_port, timeout=timeout)
        return proxy_port
      except ECPProxyError as e:
        if self.proxy_process and self.proxy_process.poll() is None:
          self.proxy_process.terminate()
        if attempt < max_retries:
          log.debug(f'ECP proxy failed to start on port {proxy_port}: {e}')
          continue
        log.debug(f'ECP proxy failed to start after {max_retries} retries: {e}')
        raise

  def _start_ecp_proxy(self, *, ecp_http_proxy: str, proxy_port: int) -> None:
    """Launches the local ECP proxy executable as a subprocess."""
    log.debug(f'Starting local ECP proxy server on port {proxy_port}')

    args = [
        '-enterprise_certificate_file_path',
        self.certificate_config_file_path,
        '-port',
        str(proxy_port),
        '-nonce_token',
        self.nonce_token,
    ]

    if self.gcloud_proxy_url:
      args.extend(
          ['-gcloud_configured_upstream_proxy_url', self.gcloud_proxy_url]
      )

    proxy_args = execution_utils.ArgsForExecutableTool(ecp_http_proxy, *args)
    try:
      self.proxy_process = subprocess.Popen(
          proxy_args,
          stdout=None,
          stderr=None,
      )
    except (OSError, ValueError) as e:
      log.debug(f'Failed to start ECP proxy executable: {e}')
      raise ECPProxyError(
          'Failed to start ECP proxy process', original_exception=e
      ) from e

  def _wait_for_proxy(self, *, proxy_port: int, timeout: int) -> None:
    """Waits for the proxy to become available and verifies its identity."""
    log.debug(f'Waiting for the proxy to be ready on port {proxy_port}...')
    if self.proxy_process is None:
      raise ECPProxyError('Proxy process has not been started.')

    start_time = time.monotonic()

    while time.monotonic() - start_time < timeout:
      if self.proxy_process.poll() is not None:
        raise ECPProxyError(
            'Proxy process terminated unexpectedly with code '
            f'{self.proxy_process.returncode} while waiting for it to start.'
        )

      try:
        with socket.create_connection(
            (self.proxy_host, proxy_port), timeout=0.1
        ):
          break
      except OSError:
        time.sleep(0.1)
        continue
    else:
      self.close()
      raise ECPProxyError(
          f'ECP Proxy on {self.proxy_host}:{proxy_port} did not become ready in'
          f' {timeout} seconds.'
      )

    try:
      # pylint:disable=g-import-not-at-top
      # local import to avoid circular dependency
      import requests as pip_requests

      readyz_url = f'http://{self.proxy_host}:{proxy_port}/readyz'
      response = pip_requests.get(readyz_url, timeout=1)
      if response.status_code != 200:
        raise ECPProxyError(
            f'Proxy /readyz endpoint returned status {response.status_code}.'
        )

      server_nonce = response.text
      if server_nonce != self.nonce_token:
        raise ECPProxyError('Nonce mismatch from proxy /readyz endpoint.')

      log.debug('Proxy is ready and nonce verified.')
    except Exception as e:
      raise ECPProxyError(
          'Failed to verify proxy readiness via /readyz endpoint.',
          original_exception=e,
      ) from e

  def close(self):
    """Terminates the background ECP proxy process to clean up resources."""
    log.debug('Closing ECP Proxy Adapter and terminating proxy process...')
    if self.proxy_process and self.proxy_process.poll() is None:
      self.proxy_process.terminate()
      try:
        self.proxy_process.wait(timeout=0.5)
      except subprocess.TimeoutExpired:
        log.debug('Proxy process did not terminate gracefully, killing it.')
        self.proxy_process.kill()


def get_proxy_port() -> Optional[int]:
  """Retrieves the local ECP HTTP Proxy port, starting it if required.

  If the proxy is already running (either initiated by a previous call or a
  parent process), this function immediately returns the cached port from
  the environment variable without spawning any new processes.

  If the proxy is not running, it evaluates the context-aware configuration to
  determine if an enterprise mTLS certificate offloading proxy is required.
  If required, it synchronously spawns the proxy subprocess, blocks to perform
  security nonce and readiness verification checks, and returns the bound TCP
  port once the server is fully ready.

  Returns:
    Optional[int]: The local TCP port of the running ECP HTTP Proxy server,
      or None if ECP proxy offloading is not required by configuration or
      fails to start.
  """
  global _ECP_HTTP_PROXY_MANAGER_INSTANCE

  # If the port is already set, the proxy was started by a parent process or
  # a previous call.
  port_str = encoding.GetEncodedValue(os.environ, _CLOUDSDK_ECP_HTTP_PROXY_PORT)
  if port_str:
    try:
      return int(port_str)
    except ValueError:
      pass

  # Start the local proxy server if mTLS settings require it.
  try:
    ca_config = context_aware.Config()
  except (
      context_aware.CertProvisionException,
      context_aware.ConfigException,
  ) as e:
    log.debug('Failed to load context-aware configuration: %s', e)
    return None

  if (
      ca_config
      and ca_config.config_type
      == context_aware.ConfigType.ENTERPRISE_CERTIFICATE
      and ca_config.use_local_proxy
  ):
    if _ECP_HTTP_PROXY_MANAGER_INSTANCE is None:
      try:
        _ECP_HTTP_PROXY_MANAGER_INSTANCE = _ECPHTTPProxyManager(
            certificate_config_file_path=ca_config.certificate_config_file_path
        )
      except ECPProxyError as e:
        log.error(f'Failed to start ECP HTTP Proxy: {e}')

    if _ECP_HTTP_PROXY_MANAGER_INSTANCE:
      return _ECP_HTTP_PROXY_MANAGER_INSTANCE.proxy_port

  return None



"""Generated client library for faulttesting version v1alpha."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.faulttesting.v1alpha import faulttesting_v1alpha_messages as messages


class FaulttestingV1alpha(base_api.BaseApiClient):
  """Generated client library for service faulttesting version v1alpha."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://faulttesting.googleapis.com/'
  MTLS_BASE_URL = 'https://faulttesting.mtls.googleapis.com/'

  _PACKAGE = 'faulttesting'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1alpha'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'FaulttestingV1alpha'
  _URL_VERSION = 'v1alpha'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new faulttesting handle."""
    url = url or self.BASE_URL
    super(FaulttestingV1alpha, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_exclusionWindows = self.ProjectsLocationsExclusionWindowsService(self)
    self.projects_locations_experimentTemplates = self.ProjectsLocationsExperimentTemplatesService(self)
    self.projects_locations_experiments_affectedResources = self.ProjectsLocationsExperimentsAffectedResourcesService(self)
    self.projects_locations_experiments = self.ProjectsLocationsExperimentsService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_validations_validationResources = self.ProjectsLocationsValidationsValidationResourcesService(self)
    self.projects_locations_validations = self.ProjectsLocationsValidationsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsExclusionWindowsService(base_api.BaseApiService):
    """Service class for the projects_locations_exclusionWindows resource."""

    _NAME = 'projects_locations_exclusionWindows'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsExclusionWindowsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new exclusion window. Once created, an exclusion window must be started with `StartExclusionWindow` to take effect.

      Args:
        request: (FaulttestingProjectsLocationsExclusionWindowsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExclusionWindow) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/exclusionWindows',
        http_method='POST',
        method_id='faulttesting.projects.locations.exclusionWindows.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['exclusionWindowId'],
        relative_path='v1alpha/{+parent}/exclusionWindows',
        request_field='exclusionWindow',
        request_type_name='FaulttestingProjectsLocationsExclusionWindowsCreateRequest',
        response_type_name='ExclusionWindow',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes an exclusion window.

      Args:
        request: (FaulttestingProjectsLocationsExclusionWindowsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/exclusionWindows/{exclusionWindowsId}',
        http_method='DELETE',
        method_id='faulttesting.projects.locations.exclusionWindows.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExclusionWindowsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Get the details of an exclusion window.

      Args:
        request: (FaulttestingProjectsLocationsExclusionWindowsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExclusionWindow) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/exclusionWindows/{exclusionWindowsId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.exclusionWindows.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExclusionWindowsGetRequest',
        response_type_name='ExclusionWindow',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all exclusion windows in a project and region, or all exclusion windows in a project in all regions if the location is given as "-".

      Args:
        request: (FaulttestingProjectsLocationsExclusionWindowsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListExclusionWindowsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/exclusionWindows',
        http_method='GET',
        method_id='faulttesting.projects.locations.exclusionWindows.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/exclusionWindows',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExclusionWindowsListRequest',
        response_type_name='ListExclusionWindowsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an exclusion window.

      Args:
        request: (FaulttestingProjectsLocationsExclusionWindowsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExclusionWindow) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/exclusionWindows/{exclusionWindowsId}',
        http_method='PATCH',
        method_id='faulttesting.projects.locations.exclusionWindows.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='exclusionWindow',
        request_type_name='FaulttestingProjectsLocationsExclusionWindowsPatchRequest',
        response_type_name='ExclusionWindow',
        supports_download=False,
    )

    def Start(self, request, global_params=None):
      r"""Starts or restarts an existing exclusion window. This method sets `start_time` to the current time and `end_time` to current time plus the duration.

      Args:
        request: (FaulttestingProjectsLocationsExclusionWindowsStartRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExclusionWindow) The response message.
      """
      config = self.GetMethodConfig('Start')
      return self._RunMethod(
          config, request, global_params=global_params)

    Start.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/exclusionWindows/{exclusionWindowsId}:start',
        http_method='POST',
        method_id='faulttesting.projects.locations.exclusionWindows.start',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:start',
        request_field='startExclusionWindowRequest',
        request_type_name='FaulttestingProjectsLocationsExclusionWindowsStartRequest',
        response_type_name='ExclusionWindow',
        supports_download=False,
    )

    def Stop(self, request, global_params=None):
      r"""Stops an active exclusion window. This method sets `end_time` to the current time.

      Args:
        request: (FaulttestingProjectsLocationsExclusionWindowsStopRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExclusionWindow) The response message.
      """
      config = self.GetMethodConfig('Stop')
      return self._RunMethod(
          config, request, global_params=global_params)

    Stop.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/exclusionWindows/{exclusionWindowsId}:stop',
        http_method='POST',
        method_id='faulttesting.projects.locations.exclusionWindows.stop',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:stop',
        request_field='stopExclusionWindowRequest',
        request_type_name='FaulttestingProjectsLocationsExclusionWindowsStopRequest',
        response_type_name='ExclusionWindow',
        supports_download=False,
    )

  class ProjectsLocationsExperimentTemplatesService(base_api.BaseApiService):
    """Service class for the projects_locations_experimentTemplates resource."""

    _NAME = 'projects_locations_experimentTemplates'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsExperimentTemplatesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new experiment template. You can use this template to create multiple `Experiment` instances.

      Args:
        request: (FaulttestingProjectsLocationsExperimentTemplatesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExperimentTemplate) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experimentTemplates',
        http_method='POST',
        method_id='faulttesting.projects.locations.experimentTemplates.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['experimentTemplateId'],
        relative_path='v1alpha/{+parent}/experimentTemplates',
        request_field='experimentTemplate',
        request_type_name='FaulttestingProjectsLocationsExperimentTemplatesCreateRequest',
        response_type_name='ExperimentTemplate',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes an experiment template.

      Args:
        request: (FaulttestingProjectsLocationsExperimentTemplatesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experimentTemplates/{experimentTemplatesId}',
        http_method='DELETE',
        method_id='faulttesting.projects.locations.experimentTemplates.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentTemplatesDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Retrieves the details of a specific experiment template.

      Args:
        request: (FaulttestingProjectsLocationsExperimentTemplatesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExperimentTemplate) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experimentTemplates/{experimentTemplatesId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.experimentTemplates.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentTemplatesGetRequest',
        response_type_name='ExperimentTemplate',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all experiment templates in a specified project and location.

      Args:
        request: (FaulttestingProjectsLocationsExperimentTemplatesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListExperimentTemplatesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experimentTemplates',
        http_method='GET',
        method_id='faulttesting.projects.locations.experimentTemplates.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/experimentTemplates',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentTemplatesListRequest',
        response_type_name='ListExperimentTemplatesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an existing experiment template.

      Args:
        request: (FaulttestingProjectsLocationsExperimentTemplatesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ExperimentTemplate) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experimentTemplates/{experimentTemplatesId}',
        http_method='PATCH',
        method_id='faulttesting.projects.locations.experimentTemplates.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='experimentTemplate',
        request_type_name='FaulttestingProjectsLocationsExperimentTemplatesPatchRequest',
        response_type_name='ExperimentTemplate',
        supports_download=False,
    )

  class ProjectsLocationsExperimentsAffectedResourcesService(base_api.BaseApiService):
    """Service class for the projects_locations_experiments_affectedResources resource."""

    _NAME = 'projects_locations_experiments_affectedResources'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsExperimentsAffectedResourcesService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets the details of a specific cloud resource that is being or was affected by a particular `Experiment`. The response includes the resource's state within the `Experiment`.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsAffectedResourcesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AffectedResource) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}/affectedResources/{affectedResourcesId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.experiments.affectedResources.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentsAffectedResourcesGetRequest',
        response_type_name='AffectedResource',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all cloud resources that are being or were affected by a specific `Experiment`. This method is useful for seeing the scope of an `Experiment` and the status of each resource involved.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsAffectedResourcesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListAffectedResourcesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}/affectedResources',
        http_method='GET',
        method_id='faulttesting.projects.locations.experiments.affectedResources.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/affectedResources',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentsAffectedResourcesListRequest',
        response_type_name='ListAffectedResourcesResponse',
        supports_download=False,
    )

  class ProjectsLocationsExperimentsService(base_api.BaseApiService):
    """Service class for the projects_locations_experiments resource."""

    _NAME = 'projects_locations_experiments'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsExperimentsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates an `Experiment` and prepares it to run by identifying all cloud resources that will be affected. The `Experiment` starts in the `PREPARING` state and transitions to `PREPARED` once the resources are identified. You can then list these resources using the [`list` method for `AffectedResources`](list.html). To execute the `Experiment`, call [`start`](start.html) on the `Experiment` after it reaches the `PREPARED` state.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Experiment) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments',
        http_method='POST',
        method_id='faulttesting.projects.locations.experiments.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['experimentId'],
        relative_path='v1alpha/{+parent}/experiments',
        request_field='experiment',
        request_type_name='FaulttestingProjectsLocationsExperimentsCreateRequest',
        response_type_name='Experiment',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes an `Experiment` record. The `Experiment` must be in a `COMPLETED` state before it can be deleted.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}',
        http_method='DELETE',
        method_id='faulttesting.projects.locations.experiments.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Returns a single `Experiment` resource, including its current state and configuration.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Experiment) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.experiments.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentsGetRequest',
        response_type_name='Experiment',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all experiments in a specified project and location.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListExperimentsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments',
        http_method='GET',
        method_id='faulttesting.projects.locations.experiments.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/experiments',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsExperimentsListRequest',
        response_type_name='ListExperimentsResponse',
        supports_download=False,
    )

    def Start(self, request, global_params=None):
      r"""Starts an existing `Experiment`. The `Experiment` must already be in the `PREPARED` state. This method initiates the fault injection process. The `Experiment` state will transition to `INJECTING` and then to `INJECTED` once the fault is active.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsStartRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Experiment) The response message.
      """
      config = self.GetMethodConfig('Start')
      return self._RunMethod(
          config, request, global_params=global_params)

    Start.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}:start',
        http_method='POST',
        method_id='faulttesting.projects.locations.experiments.start',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:start',
        request_field='startExperimentRequest',
        request_type_name='FaulttestingProjectsLocationsExperimentsStartRequest',
        response_type_name='Experiment',
        supports_download=False,
    )

    def Stop(self, request, global_params=None):
      r"""Stops a running `Experiment`. This method initiates the process of reverting any changes made by the `Experiment`, returning the affected resources to their normal state. The `Experiment` state will transition to `REVERTING` and then to `COMPLETED`.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsStopRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Experiment) The response message.
      """
      config = self.GetMethodConfig('Stop')
      return self._RunMethod(
          config, request, global_params=global_params)

    Stop.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments/{experimentsId}:stop',
        http_method='POST',
        method_id='faulttesting.projects.locations.experiments.stop',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:stop',
        request_field='stopExperimentRequest',
        request_type_name='FaulttestingProjectsLocationsExperimentsStopRequest',
        response_type_name='Experiment',
        supports_download=False,
    )

    def StopAll(self, request, global_params=None):
      r"""Stops all running experiments in a given location for a given project. This method initiates the process of reverting any changes made by the experiments, returning the affected resources to their normal state. The `Experiment` states will transition to `REVERTING` and then to `COMPLETED`. Returns an RPC error if it can't list experiments at all. Otherwise, it lists and stops all experiments it can, and returns a response including success count and a list of errors. If there are no experiments to stop, it returns successfully with an empty response message.

      Args:
        request: (FaulttestingProjectsLocationsExperimentsStopAllRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (StopAllExperimentsResponse) The response message.
      """
      config = self.GetMethodConfig('StopAll')
      return self._RunMethod(
          config, request, global_params=global_params)

    StopAll.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/experiments:stopAll',
        http_method='POST',
        method_id='faulttesting.projects.locations.experiments.stopAll',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1alpha/{+parent}/experiments:stopAll',
        request_field='stopAllExperimentsRequest',
        request_type_name='FaulttestingProjectsLocationsExperimentsStopAllRequest',
        response_type_name='StopAllExperimentsResponse',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of `1`, corresponding to `Code.CANCELLED`.

      Args:
        request: (FaulttestingProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='faulttesting.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='FaulttestingProjectsLocationsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (FaulttestingProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='faulttesting.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (FaulttestingProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`.

      Args:
        request: (FaulttestingProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='faulttesting.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken', 'returnPartialSuccess'],
        relative_path='v1alpha/{+name}/operations',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsValidationsValidationResourcesService(base_api.BaseApiService):
    """Service class for the projects_locations_validations_validationResources resource."""

    _NAME = 'projects_locations_validations_validationResources'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsValidationsValidationResourcesService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets the details of a specific `ValidationResource`. This field shows a resource that was flagged as potentially affected by the `Action` defined in the parent `Validation`.

      Args:
        request: (FaulttestingProjectsLocationsValidationsValidationResourcesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ValidationResource) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations/{validationsId}/validationResources/{validationResourcesId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.validations.validationResources.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsValidationsValidationResourcesGetRequest',
        response_type_name='ValidationResource',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all cloud resources identified as potentially affected within a specific `Validation` dry run.

      Args:
        request: (FaulttestingProjectsLocationsValidationsValidationResourcesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListValidationResourcesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations/{validationsId}/validationResources',
        http_method='GET',
        method_id='faulttesting.projects.locations.validations.validationResources.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/validationResources',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsValidationsValidationResourcesListRequest',
        response_type_name='ListValidationResourcesResponse',
        supports_download=False,
    )

  class ProjectsLocationsValidationsService(base_api.BaseApiService):
    """Service class for the projects_locations_validations resource."""

    _NAME = 'projects_locations_validations'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsValidationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new `Validation` dry run. This process involves analyzing the provided `Action` to identify all the Google Cloud resources that match the targeting criteria. You can view the results by listing the `ValidationResource` children of the `Validation` dry run. The `Validation` dry run starts in the `PREPARING` state and transitions to `COMPLETED` once the analysis is finished.

      Args:
        request: (FaulttestingProjectsLocationsValidationsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Validation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations',
        http_method='POST',
        method_id='faulttesting.projects.locations.validations.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['validationId'],
        relative_path='v1alpha/{+parent}/validations',
        request_field='validation',
        request_type_name='FaulttestingProjectsLocationsValidationsCreateRequest',
        response_type_name='Validation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a record of a dry run.

      Args:
        request: (FaulttestingProjectsLocationsValidationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations/{validationsId}',
        http_method='DELETE',
        method_id='faulttesting.projects.locations.validations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsValidationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Retrieves the details of a specific `Validation` dry run.

      Args:
        request: (FaulttestingProjectsLocationsValidationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Validation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations/{validationsId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.validations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsValidationsGetRequest',
        response_type_name='Validation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all dry runs in a specified project and location.

      Args:
        request: (FaulttestingProjectsLocationsValidationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListValidationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/validations',
        http_method='GET',
        method_id='faulttesting.projects.locations.validations.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/validations',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsValidationsListRequest',
        response_type_name='ListValidationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (FaulttestingProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='faulttesting.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the ListLocationsRequest.name field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (FaulttestingProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations',
        http_method='GET',
        method_id='faulttesting.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+name}/locations',
        request_field='',
        request_type_name='FaulttestingProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(FaulttestingV1alpha.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""'Bare Metal Solution os images list command."""


import abc

from googlecloudsdk.api_lib.bms.bms_client import BmsClient
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.bms import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties


DETAILED_HELP = {
    'DESCRIPTION':
        """
          List Bare Metal Solution OS images in a project.
        """,
    'EXAMPLES':
        """
          To list all OS images within the project, run:

            $ {command}
    """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class List(base.CacheCommand, metaclass=abc.ABCMeta):
  """List Bare Metal Solution OS images in a project."""

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    flags.FILTER_FLAG_NO_SORTBY_DOC.AddToParser(parser)
    flags.LIMIT_FLAG_NO_SORTBY_DOC.AddToParser(parser)
    parser.display_info.AddFormat('table(code, description)')

  def Run(self, args):
    client = BmsClient()
    project = properties.VALUES.core.project.Get(required=True)
    return client.ListOSImages(project_resource=project, limit=args.limit)

  def Epilog(self, resources_were_displayed):
    """Called after resources are displayed if the default format was used.

    Args:
      resources_were_displayed: True if resources were displayed.
    """
    if not resources_were_displayed:
      log.status.Print('Listed 0 items.')


List.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Open Console command."""


import urllib.parse

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.app import browser_dispatcher
from googlecloudsdk.core import properties


CONSOLE_URL = 'https://console.developers.google.com/appengine?{query}'
LOGS_URL = 'https://console.developers.google.com/logs?{query}'


def _CreateDevConsoleURL(project, service='default', version=None, logs=False):
  """Creates a URL to a page in the Developer Console according to the params.

  Args:
    project: The app engine project id
    service: A service belonging to the project
    version: A version belonging to the service, or all versions if omitted
    logs: If true, go to the log section instead of the dashboard
  Returns:
    The URL to the respective page in the Developer Console
  """
  if service is None:
    service = 'default'
  query = [('project', project), ('serviceId', service)]
  if version:
    query.append(('versionId', version))
  query_string = urllib.parse.urlencode(query)
  return (LOGS_URL if logs else CONSOLE_URL).format(query=query_string)


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
class OpenConsole(base.Command):
  """Open the App Engine dashboard, or log viewer, in a web browser.

  """

  detailed_help = {
      'DESCRIPTION': """\
          {description}
          """,
      'EXAMPLES': """\
          Open the App Engine dashboard for the default service:

              $ {command}

          Open the service specific dashboard view:

              $ {command} --service="myService"

          Open the version specific dashboard view:

              $ {command} --service="myService" --version="v1"

          Open the log viewer for the default service:

              $ {command} --logs
          """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        '--service',
        '-s',
        help=('The service to consider. If not specified, use the '
              'default service.'))
    parser.add_argument(
        '--version',
        '-v',
        help=('The version to consider. If not specified, '
              'all versions for the given service are considered.'))
    parser.add_argument(
        '--logs',
        '-l',
        action='store_true',
        default=False,
        help='Open the log viewer instead of the App Engine dashboard.')

  def Run(self, args):
    project = properties.VALUES.core.project.Get(required=True)
    url = _CreateDevConsoleURL(project, args.service, args.version, args.logs)
    browser_dispatcher.OpenURL(url)



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Backup and DR Backup plan associations client."""

from googlecloudsdk.api_lib.backupdr import util
from googlecloudsdk.core import properties


class DataSourceReferencesClient(util.BackupDrClientBase):
  """Cloud Backup and DR Data Source References client."""

  def __init__(self, api_version=util.DEFAULT_API_VERSION):
    super(DataSourceReferencesClient, self).__init__(api_version=api_version)
    self.service = self.client.projects_locations_dataSourceReferences

  def FetchForResourceType(
      self,
      location,
      resource_type,
      filter_expression=None,
      page_size=None,
      order_by=None,
  ):
    project = properties.VALUES.core.project.GetOrFail()
    parent = 'projects/{}/locations/{}'.format(project, location)
    request = self.messages.BackupdrProjectsLocationsDataSourceReferencesFetchForResourceTypeRequest(
        parent=parent,
        resourceType=resource_type,
        pageSize=page_size,
        filter=filter_expression,
        orderBy=order_by,
    )
    return self.service.FetchForResourceType(request)



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a new Cloud SQL instance."""
# TODO(b/362481808): remove python2-isms.

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.sql import api_util as common_api_util
from googlecloudsdk.api_lib.sql import exceptions as sql_exceptions
from googlecloudsdk.api_lib.sql import operations
from googlecloudsdk.api_lib.sql import validate
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.kms import resource_args as kms_resource_args
from googlecloudsdk.command_lib.sql import flags
from googlecloudsdk.command_lib.sql import instances as command_util
from googlecloudsdk.command_lib.sql import validate as command_validate
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.resource import resource_lex
from googlecloudsdk.core.resource import resource_property

# 1h, based off of the max time it usually takes to create a SQL instance.
_INSTANCE_CREATION_TIMEOUT_SECONDS = 3600

DETAILED_HELP = {
    'EXAMPLES': """\
        To create a MySQL 8.0 instance with ID ``prod-instance'' that has 2
        CPUs, 4 GB of RAM, and is in the region ``us-central1'' (a zone will be
        auto-assigned), where the 'root' user has its password set to
        ``password123'', run:

          $ {command} prod-instance --database-version=MYSQL_8_0 --cpu=2 --memory=4GB --region=us-central1 --root-password=password123

        To create a Postgres 15 instance with ID ``prod-instance'' that has 2
        CPUs, 8 GiB of RAM, and is in the zone ``us-central1-a'', where the
        'postgres' user has its password set to ``password123'', run:

          $ {command} prod-instance --database-version=POSTGRES_15 --cpu=2 --memory=8GiB --zone=us-central1-a --root-password=password123

        To create a SQL Server 2022 Express instance with ID ``prod-instance''
        that has 2 CPUs, 3840MiB of RAM, and is in the zone ``us-central1-a'',
        where the 'sqlserver' user has its password set to ``password123'',
        run:

          $ {command} prod-instance --database-version=SQLSERVER_2022_EXPRESS --cpu=2 --memory=3840MiB --zone=us-central1-a --root-password=password123
        """,
}


def AddBaseArgs(parser):
  """Declare flag and positional arguments for this command parser."""
  # TODO(b/35705305): move common flags to command_lib.sql.flags
  base.ASYNC_FLAG.AddToParser(parser)
  parser.display_info.AddFormat(flags.GetInstanceListFormat())
  flags.AddActivationPolicy(parser)
  flags.AddActiveDirectoryDomain(parser)
  flags.AddAssignIp(parser)
  flags.AddAuthorizedNetworks(parser)
  flags.AddAvailabilityType(parser)
  flags.AddBackup(parser)
  flags.AddBackupStartTime(parser)
  flags.AddBackupLocation(parser, allow_empty=False)
  flags.AddCPU(parser)
  flags.AddInstanceCollation(parser)
  flags.AddDatabaseFlags(parser)
  flags.AddEnableBinLog(parser, show_negated_in_help=False)
  flags.AddRetainedBackupsCount(parser)
  flags.AddRetainedTransactionLogDays(parser)
  flags.AddFailoverReplicaName(parser)
  parser.add_argument(
      'instance',
      type=command_validate.InstanceNameRegexpValidator(),
      help='Cloud SQL instance ID.',
  )
  flags.AddMaintenanceReleaseChannel(parser)
  flags.AddMaintenanceWindowDay(parser)
  flags.AddMaintenanceWindowHour(parser)
  flags.AddDenyMaintenancePeriodStartDate(parser)
  flags.AddDenyMaintenancePeriodEndDate(parser)
  flags.AddDenyMaintenancePeriodTime(parser)
  flags.AddInsightsConfigQueryInsightsEnabled(parser, show_negated_in_help=True)
  flags.AddInsightsConfigEnhancedQueryInsightsEnabled(
      parser, show_negated_in_help=True
  )
  flags.AddInsightsConfigQueryStringLength(parser)
  flags.AddInsightsConfigRecordApplicationTags(
      parser, show_negated_in_help=True
  )
  flags.AddInsightsConfigRecordClientAddress(parser, show_negated_in_help=True)
  flags.AddInsightsConfigQueryPlansPerMinute(parser)
  flags.AddMasterInstanceName(parser)
  flags.AddMemory(parser)
  flags.AddPasswordPolicyMinLength(parser)
  flags.AddPasswordPolicyComplexity(parser)
  flags.AddPasswordPolicyReuseInterval(parser)
  flags.AddPasswordPolicyDisallowUsernameSubstring(parser)
  flags.AddPasswordPolicyPasswordChangeInterval(parser)
  flags.AddPasswordPolicyEnablePasswordPolicy(parser)
  flags.AddReplicaType(parser)
  flags.AddReplication(parser)
  flags.AddRequireSsl(parser)
  flags.AddRootPassword(parser)
  flags.AddStorageAutoIncrease(parser)
  flags.AddStorageSize(parser)
  flags.AddStorageType(parser)
  flags.AddTier(parser)
  flags.AddEdition(parser)
  kms_flag_overrides = {
      'kms-key': '--disk-encryption-key',
      'kms-keyring': '--disk-encryption-key-keyring',
      'kms-location': '--disk-encryption-key-location',
      'kms-project': '--disk-encryption-key-project',
  }
  kms_resource_args.AddKmsKeyResourceArg(
      parser, 'instance', flag_overrides=kms_flag_overrides
  )
  flags.AddEnablePointInTimeRecovery(parser)
  flags.AddNetwork(parser)
  flags.AddSqlServerAudit(parser)
  flags.AddDeletionProtection(parser)
  flags.AddSqlServerTimeZone(parser)
  flags.AddConnectorEnforcement(parser)
  flags.AddTimeout(parser, _INSTANCE_CREATION_TIMEOUT_SECONDS)
  flags.AddEnableGooglePrivatePath(parser, show_negated_in_help=False)
  flags.AddThreadsPerCore(parser)
  flags.AddCascadableReplica(parser)
  flags.AddEnableDataCache(parser)
  flags.AddEnableAutoUpgrade(parser)
  flags.AddRecreateReplicasOnPrimaryCrash(parser)
  psc_setup_group = parser.add_group()
  flags.AddEnablePrivateServiceConnect(psc_setup_group)
  flags.AddAllowedPscProjects(psc_setup_group)
  flags.AddCustomSubjectAlternativeNames(parser)
  flags.AddSslMode(parser)
  flags.AddEnableGoogleMLIntegration(parser)
  flags.AddEnableDataplexIntegration(parser)
  flags.AddPscAutoConnections(parser)
  flags.AddServerCaMode(parser)
  flags.AddTags(parser)
  flags.AddRetainBackupsOnDelete(parser)
  flags.AddServerCaPool(parser)
  flags.AddStorageProvisionedIops(parser)
  flags.AddStorageProvisionedThroughput(parser)
  flags.AddInstanceType(parser)
  flags.AddNodeCount(parser)
  flags.AddActiveDirectoryMode(parser)
  flags.AddActiveDirectorySecretManagerKey(parser)
  flags.AddActiveDirectoryOrganizationalUnit(parser)
  flags.AddActiveDirectoryDNSServers(parser)
  flags.ClearActiveDirectoryDNSServers(parser)
  flags.AddForceSqlNetworkArchitecture(parser)
  flags.AddFinalBackup(parser)
  flags.AddFinalbackupRetentionDays(parser)
  flags.AddServerCertificateRotationMode(parser)
  flags.AddEnableConnectionPooling(parser)
  flags.AddConnectionPoolFlags(parser)
  flags.AddReadPoolAutoScaleConfig(parser)
  flags.AddDataApiAccess(parser)
  flags.AddSqlServerEntraId(parser)
  flags.AddPerformanceCaptureConfig(parser)
  flags.AddEnablePscAutoConnectionPolicy(parser)

  # When adding a new field for instance creation, determine if it should also
  # be included in the restore to new instance command. This command uses backup
  # settings to create a new instance, allowing users to override some settings.
  # If the new field should be user-overridable during restore, add it to the
  # restore command.


def AddBetaArgs(parser):
  """Declare beta flag and positional arguments for this command parser."""
  flags.AddExternalMasterGroup(parser)
  flags.AddInstanceResizeLimit(parser)
  flags.AddAllocatedIpRangeName(parser)
  labels_util.AddCreateLabelsFlags(parser)
  flags.AddReplicationLagMaxSecondsForRecreate(parser)
  flags.AddEnableDbAlignedAtomicWrites(parser)
  flags.AddEnableAcceleratedReplicaMode(parser)
  flags.AddUncMappings(parser)
  flags.AddEnablePscAutoDns(parser)
  flags.AddEnablePscWriteEndpointDns(parser)


def AddAlphaArgs(parser):
  """Declare alpha flags for this command parser."""
  parser.add_argument(
      '--enable-confidential-storage',
      action='store_true',
      default=None,
      hidden=True,
      help='Whether to enable confidential mode for the instance.',
  )
  flags.AddMsdtcEnabled(parser, hidden=True)
  flags.AddMsdtcRpcAuthMode(parser, hidden=True)
  flags.AddMsdtcHostMappings(parser, hidden=True)


def RunBaseCreateCommand(args, release_track):
  """Creates a new Cloud SQL instance.

  Args:
    args: argparse.Namespace, The arguments that this command was invoked with.
    release_track: base.ReleaseTrack, the release track that this was run under.

  Returns:
    A dict object representing the operations resource describing the create
    operation if the create was successful.
  Raises:
    HttpException: A http error response was received while executing api
        request.
    ArgumentError: An argument supplied by the user was incorrect, such as
      specifying an invalid CMEK configuration or attempting to create a V1
      instance.
    RequiredArgumentException: A required argument was not supplied by the user,
      such as omitting --root-password on a SQL Server instance.
  """
  client = common_api_util.SqlClient(common_api_util.API_VERSION_DEFAULT)
  sql_client = client.sql_client
  sql_messages = client.sql_messages

  validate.ValidateInstanceName(args.instance)
  validate.ValidateInstanceLocation(args)
  instance_ref = client.resource_parser.Parse(
      args.instance,
      params={'project': properties.VALUES.core.project.GetOrFail},
      collection='sql.instances',
  )

  # Get the region, tier, and database version from the master if these fields
  # are not specified.
  # TODO(b/64266672): Remove once API does not require these fields.
  if args.IsSpecified('master_instance_name'):
    master_instance_ref = client.resource_parser.Parse(
        args.master_instance_name,
        params={'project': properties.VALUES.core.project.GetOrFail},
        collection='sql.instances',
    )
    try:
      master_instance_resource = sql_client.instances.Get(
          sql_messages.SqlInstancesGetRequest(
              project=instance_ref.project,
              instance=master_instance_ref.instance,
          )
      )
    except apitools_exceptions.HttpError as error:
      # TODO(b/64292220): Remove once API gives helpful error message.
      log.debug('operation : %s', str(master_instance_ref))
      exc = exceptions.HttpException(error)
      if (
          resource_property.Get(
              exc.payload.content,
              resource_lex.ParseKey('error.errors[0].reason'),
              None,
          )
          == 'notAuthorized'
      ):
        msg = (
            'You are either not authorized to access the master instance or '
            'it does not exist.'
        )
        raise exceptions.HttpException(msg)
      raise
    if not args.IsSpecified('region'):
      args.region = master_instance_resource.region
    if not args.IsSpecified('database_version'):
      args.database_version = master_instance_resource.databaseVersion.name
    if (
        not args.IsSpecified('tier')
        and not (args.IsSpecified('cpu') or args.IsSpecified('memory'))
        and master_instance_resource.settings
    ):
      args.tier = master_instance_resource.settings.tier

    if args.IsKnownAndSpecified('enable_accelerated_replica_mode'):
      if not (
          master_instance_resource.databaseVersion.name.startswith('MYSQL_')
      ):
        raise sql_exceptions.ArgumentError(
            '--enable-accelerated-replica-mode is only supported for MySQL.'
        )

    # Validate master/replica CMEK configurations.
    if master_instance_resource.diskEncryptionConfiguration:
      if args.region == master_instance_resource.region:
        # Warn user that same-region replicas inherit their master's CMEK
        # configuration.
        command_util.ShowCmekWarning('replica', 'the master instance')
      elif not args.IsSpecified('disk_encryption_key'):
        # Raise error that cross-region replicas require their own CMEK key if
        # the master is CMEK.
        raise exceptions.RequiredArgumentException(
            '--disk-encryption-key',
            '`--disk-encryption-key` is required when creating a cross-region '
            'replica of an instance with customer-managed encryption.',
        )
      else:
        command_util.ShowCmekWarning('replica')
    elif args.IsSpecified('disk_encryption_key'):
      # Raise error that cross-region replicas cannot be CMEK encrypted if their
      # master is not.
      raise sql_exceptions.ArgumentError(
          '`--disk-encryption-key` cannot be specified when creating a replica '
          'of an instance without customer-managed encryption.'
      )

    if args.IsSpecified('cascadable_replica'):
      if args.region == master_instance_resource.region:
        raise exceptions.InvalidArgumentException(
            '--cascadable-replica',
            '`--cascadable-replica` can only be specified when creating a '
            'replica that is in a different region than the primary.',
        )
  else:
    if args.IsSpecified('cascadable_replica'):
      raise exceptions.InvalidArgumentException(
          '--cascadable-replica',
          '`--cascadable-replica` can only be specified when '
          '`--master-instance-name` is specified.',
      )

    if args.IsKnownAndSpecified('enable_accelerated_replica_mode'):
      if args.IsSpecified('database_version'):
        if not args.database_version.startswith('MYSQL_'):
          raise sql_exceptions.ArgumentError(
              '--enable-accelerated-replica-mode is only supported for MySQL.'
          )

  # --root-password is required when creating SQL Server instances
  if (
      args.IsSpecified('database_version')
      and args.database_version.startswith('SQLSERVER')
      and not args.IsSpecified('root_password')
  ):
    raise exceptions.RequiredArgumentException(
        '--root-password',
        '`--root-password` is required when creating SQL Server instances.',
    )

  if not args.backup:
    if args.IsSpecified('enable_bin_log'):
      if not args.IsSpecified('master_instance_name'):
        # if we are creating a replica, we allow enable_bin_log with no_backup,
        # otherwise no_backup is not allowed with enable_bin_log
        raise sql_exceptions.ArgumentError(
            '`--enable-bin-log` cannot be specified when --no-backup is '
            'specified'
        )
    elif args.IsSpecified('enable_point_in_time_recovery'):
      raise sql_exceptions.ArgumentError(
          '`--enable-point-in-time-recovery` cannot be specified when '
          '--no-backup is specified'
      )

  if args.IsKnownAndSpecified(
      'allowed_psc_projects'
  ) and not args.IsKnownAndSpecified('enable_private_service_connect'):
    raise sql_exceptions.ArgumentError(
        '`--allowed-psc-projects` requires `--enable-private-service-connect`'
    )

  if args.IsKnownAndSpecified(
      'psc_auto_connections'
  ) and not args.IsKnownAndSpecified('enable_private_service_connect'):
    raise sql_exceptions.ArgumentError(
        '`--psc-auto-connections` requires `--enable-private-service-connect`'
    )

  if args.IsKnownAndSpecified(
      'enable_psc_auto_dns'
  ) and not args.IsKnownAndSpecified('enable_private_service_connect'):
    raise sql_exceptions.ArgumentError(
        '`--enable-psc-auto-dns` requires `--enable-private-service-connect`'
    )

  if args.IsKnownAndSpecified(
      'enable_psc_write_endpoint_dns'
  ) and not args.IsKnownAndSpecified('enable_private_service_connect'):
    raise sql_exceptions.ArgumentError(
        '`--enable-psc-write-endpoint-dns` requires'
        ' `--enable-private-service-connect`'
    )

  if args.IsKnownAndSpecified(
      'enable_psc_auto_connection_policy'
  ) and not args.IsKnownAndSpecified('enable_private_service_connect'):
    raise sql_exceptions.ArgumentError(
        '`--enable-psc-auto-connection-policy` requires'
        ' `--enable-private-service-connect`'
    )

  if args.IsKnownAndSpecified(
      'enable_psc_write_endpoint_dns'
  ) and not args.IsKnownAndSpecified('enable_psc_auto_dns'):
    raise sql_exceptions.ArgumentError(
        '`--enable-psc-write-endpoint-dns` requires `--enable-psc-auto-dns`'
    )

  # TODO(b/372040396): Add the check for server_ca_mode when it is supported.
  if args.IsKnownAndSpecified(
      'custom_subject_alternative_names'
  ) and not args.IsKnownAndSpecified('server_ca_mode'):
    raise sql_exceptions.ArgumentError(
        '`--custom-subject-alternative-names` requires customer managed'
        ' server CA'
    )

  if args.database_flags is not None and any([
      'sync_binlog' in args.database_flags,
      'innodb_flush_log_at_trx_commit' in args.database_flags,
  ]):
    log.warning(
        'Changing innodb_flush_log_at_trx_commit '
        'or sync_binlog may cause data loss. Check '
        'https://cloud.google.com/sql/docs/mysql/flags'
        ' for more details.'
    )

  if (
      args.IsKnownAndSpecified('performance_capture_config')
      and args.IsSpecified('database_version')
      and not args.database_version.startswith('MYSQL')
  ):
    raise sql_exceptions.ArgumentError(
        '`--performance-capture-config` is only supported for MySQL.'
    )

  if args.IsSpecified('edition') and args.edition == 'enterprise-plus':
    # This logic is awkward because we only want to change behavior for "new"
    # database types to avoid breaking changes.
    can_infer_tier = (
        args.tier
        and command_util.DoesEnterprisePlusReplicaInferTierForDatabaseType(
            args.database_version
        )
    )
    if not (args.IsSpecified('tier') or can_infer_tier):
      raise sql_exceptions.ArgumentError(
          '`--edition=enterprise-plus` requires `--tier`'
      )

  instance_resource = (
      command_util.InstancesV1Beta4.ConstructCreateInstanceFromArgs(
          sql_messages,
          args,
          instance_ref=instance_ref,
          release_track=release_track,
      )
  )

  operation_ref = None
  try:
    result_operation = sql_client.instances.Insert(
        sql_messages.SqlInstancesInsertRequest(
            databaseInstance=instance_resource, project=instance_ref.project
        )
    )

    operation_ref = client.resource_parser.Create(
        'sql.operations',
        operation=result_operation.name,
        project=instance_ref.project,
    )

    if args.async_:
      if not args.IsSpecified('format'):
        args.format = 'default'
      return sql_client.operations.Get(
          sql_messages.SqlOperationsGetRequest(
              project=operation_ref.project, operation=operation_ref.operation
          )
      )

    operations.OperationsV1Beta4.WaitForOperation(
        sql_client,
        operation_ref,
        'Creating Cloud SQL instance for ' + args.database_version,
        max_wait_seconds=args.timeout,
    )

    log.CreatedResource(instance_ref)

    new_resource = sql_client.instances.Get(
        sql_messages.SqlInstancesGetRequest(
            project=instance_ref.project, instance=instance_ref.instance
        )
    )
    return new_resource
  except apitools_exceptions.HttpError as error:
    log.debug('operation : %s', str(operation_ref))
    exc = exceptions.HttpException(error)
    if (
        resource_property.Get(
            exc.payload.content,
            resource_lex.ParseKey('error.errors[0].reason'),
            None,
        )
        == 'errorMaxInstancePerLabel'
    ):
      msg = resource_property.Get(
          exc.payload.content, resource_lex.ParseKey('error.message'), None
      )
      raise exceptions.HttpException(msg)
    raise


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class Create(base.Command):
  """Creates a new Cloud SQL instance."""

  detailed_help = DETAILED_HELP

  def Run(self, args):
    return RunBaseCreateCommand(args, self.ReleaseTrack())

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command."""
    AddBaseArgs(parser)
    flags.AddLocationGroup(parser)
    flags.AddDatabaseVersion(parser)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
@base.DefaultUniverseOnly
class CreateBeta(base.Command):
  """Creates a new Cloud SQL instance."""

  detailed_help = DETAILED_HELP

  def Run(self, args):
    return RunBaseCreateCommand(args, self.ReleaseTrack())

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command."""
    AddBaseArgs(parser)
    flags.AddLocationGroup(parser)
    AddBetaArgs(parser)
    flags.AddDatabaseVersion(parser, restrict_choices=False)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class CreateAlpha(base.Command):
  """Creates a new Cloud SQL instance."""

  detailed_help = DETAILED_HELP

  def Run(self, args):
    return RunBaseCreateCommand(args, self.ReleaseTrack())

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command."""
    AddBaseArgs(parser)
    flags.AddLocationGroup(parser)
    AddBetaArgs(parser)
    AddAlphaArgs(parser)
    flags.AddDatabaseVersion(parser, restrict_choices=False)
